"use strict";
/**
 * Copyright ou © ou Copr. Ministère de l'Europe et des Affaires étrangères (2017)
 * <p/>
 * pole-architecture.dga-dsi-psi@diplomatie.gouv.fr
 * <p/>
 * Ce logiciel est un programme informatique servant à faciliter la création
 * d'applications Web conformément aux référentiels généraux français : RGI, RGS et RGAA
 * <p/>
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * <p/>
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * <p/>
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 * <p/>
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 * <p/>
 * <p/>
 * Copyright or © or Copr. Ministry for Europe and Foreign Affairs (2017)
 * <p/>
 * pole-architecture.dga-dsi-psi@diplomatie.gouv.fr
 * <p/>
 * This software is a computer program whose purpose is to facilitate creation of
 * web application in accordance with french general repositories : RGI, RGS and RGAA.
 * <p/>
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 * <p/>
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 * <p/>let
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 * <p/>
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * hornet-js-batch - Ensemble des composants de gestion de base hornet-js
 *
 * @author MEAE - Ministère de l'Europe et des Affaires étrangères
 * @version v5.2.3
 * @link git+https://github.com/diplomatiegouvfr/hornet-js.git
 * @license CECILL-2.1
 */
var hornet_js_utils_1 = require("hornet-js-utils");
var promise_api_1 = require("hornet-js-utils/src/promise-api");
var batch_process_1 = require("hornet-js-batch/src/core/batch-process");
var batch_status_1 = require("hornet-js-batch/src/core/batch-status");
var _ = require("lodash");
var logger = hornet_js_utils_1.Utils.getLogger("hornet-js-batch.batch-executor");
/***
 * @classdesc Classe Batch
 * elle classe permet de chainer les differents traitement du BatchUnit see{@link BatchUnit}
 * @class
 */
var Batch = /** @class */ (function () {
    function Batch(unit, roadmap) {
        if (roadmap === void 0) { roadmap = []; }
        this.unit = unit;
        this.roadmap = roadmap;
        this.currentStep = 0;
        this._id = "_" + unit.id;
        this.createDate = new Date();
    }
    Object.defineProperty(Batch.prototype, "status", {
        /**
         * renvoie le statut du batch
         */
        get: function () {
            return this._status;
        },
        /***
         * modifie le statut du batch
         */
        set: function (status) {
            this._status = status;
        },
        enumerable: true,
        configurable: true
    });
    Batch.prototype.updateStatus = function () {
        var _this = this;
        var summary = [];
        var res = this.roadmap.reduce(function (result, process) {
            var res = false;
            var status = process.status;
            summary.push({ process_name: process.name, process_status: batch_status_1.STATUS[process.status] });
            if (status === batch_status_1.STATUS.FAILED) {
                _this.endDate = new Date();
                _this.status = batch_status_1.STATUS.FAILED;
            }
            if (status === batch_status_1.STATUS.SUCCEEDED) {
                res = true;
            }
            return result && res;
        }, true);
        if (res) {
            this.endDate = new Date();
            this.status = batch_status_1.STATUS.SUCCEEDED;
        }
        logger.trace("\nBatch :", this.id, "\n", summary);
        return summary;
    };
    Object.defineProperty(Batch.prototype, "id", {
        /**
         * renvoie l'id du batch
         */
        get: function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    /***
     * cette methode permet le traitement par step.
     */
    Batch.prototype.execPool = function () {
        var _this = this;
        if (this.roadmap && this.currentStep < this.roadmap.length) {
            setImmediate(function (_batch) {
                var process = _batch.roadmap[_batch.currentStep];
                try {
                    process.execute.apply(process).then(function (result) {
                        _batch.currentStep++;
                        if (_batch.roadmap[_batch.currentStep]) {
                            _batch.roadmap[_batch.currentStep].options.args = result;
                        }
                        _batch.execPool();
                    }).catch(function (error) {
                        logger.fatal(process.name, error);
                        _this.status = batch_status_1.STATUS.FAILED;
                        _this.error = error;
                        BatchExecutor.Instance.removeBatch(_this);
                    });
                }
                catch (error) {
                    logger.fatal(process.name, error);
                    _this.status = batch_status_1.STATUS.FAILED;
                    _this.error = error;
                    BatchExecutor.Instance.removeBatch(_this);
                }
            }, this);
        }
    };
    /***
     * cette methode ordonnance puis lance les différents traitements du batch puis .
     * @return une promise du resultat des différents traitements.
     */
    Batch.prototype.run = function () {
        var _this = this;
        logger.trace("run batch " + this._id);
        if (!this.startDate) {
            this.startDate = new Date();
        }
        // normalize roadmap
        this.roadmap.forEach(function (values, index) {
            var res = values;
            if (values instanceof Array) {
                res = ((values.length > 1) ? new batch_process_1.BatchProcessParent(values) : values[0]);
            }
            res.batch = _this;
            _this.roadmap[index] = res;
        });
        this.execPool();
        return promise_api_1.Promise.resolve(this.unit);
    };
    return Batch;
}());
exports.Batch = Batch;
/***
 * @classdesc BatchExecutor
 * Classe d'orchectration des Batch
 * @class
 */
var BatchExecutor = /** @class */ (function () {
    function BatchExecutor() {
    }
    Object.defineProperty(BatchExecutor, "Instance", {
        get: function () {
            return this._instance || (this._instance = new this());
        },
        enumerable: true,
        configurable: true
    });
    /**
     *
     * @param {Batch} batch
     */
    BatchExecutor.prototype.addToSummary = function (batch) {
        if (!BatchExecutor.summary[batch.unit.route]) {
            BatchExecutor.summary[batch.unit.route] = [];
        }
        BatchExecutor.summary[batch.unit.route].unshift({
            id: batch.id,
            status: batch_status_1.STATUS[batch.status],
            startDate: batch.startDate,
            endDate: batch.endDate,
            createDate: batch.createDate,
            errorBatch: batch.error ? "[" + batch.error.name + "] " + batch.error.message : "",
        });
    };
    /***
     * Renvoie le batch associé au BatchUnit qu'il soit en cours de traitement ou dans la file d'attente.
     * S'il ne le trouve pas il en crée un qu'il place dans la file d'attente.
     * @param unit {Boolean} triggerFetch déclenche un évènement "fetch" après l'opération si true.
     * @return un batch trouvé ou un nouveau.
     */
    BatchExecutor.prototype.getBatch = function (unit) {
        var criteria = { _id: "_" + unit.id };
        var batch = _.find(BatchExecutor.processing[unit.route], criteria)
            || _.find(BatchExecutor.queue[unit.route], criteria)
            || function () {
                var newBatch = new Batch(unit);
                newBatch.status = batch_status_1.STATUS.QUEUED;
                BatchExecutor.queue[unit.route] = [newBatch];
                return newBatch;
            }();
        return batch;
    };
    /***
     * @param route une route.
     * @returns {boolean} true si une action de type batch est en cours d'execution
     */
    BatchExecutor.prototype.isBatchActionExist = function (route) {
        var test = BatchExecutor.processing[route] || BatchExecutor.queue[route];
        var found = false;
        var history = BatchExecutor.summary[route];
        if (test) {
            var batch = test[0];
            var roadmap = batch.roadmap;
            for (var i = 0; i < batch.roadmap.length; i++) {
                var item = roadmap[i];
                if (item instanceof batch_process_1.BatchProcessParent) {
                    item.updateStatus();
                }
                if (item.status === batch_status_1.STATUS.RUNNING || item.status === batch_status_1.STATUS.INIT || item.status === batch_status_1.STATUS.QUEUED) {
                    logger.trace("=> batch exist", true);
                    return {
                        isBusy: true,
                        history: history,
                    };
                }
                if (item.status === batch_status_1.STATUS.FAILED) {
                    found = true;
                }
            }
            if (found) {
                logger.trace("=> caution previous batch has failed");
                batch.status = batch_status_1.STATUS.FAILED;
                BatchExecutor.Instance.removeBatch(batch);
                test = undefined;
            }
            else {
                batch.status = batch_status_1.STATUS.SUCCEEDED;
                BatchExecutor.Instance.removeBatch(batch);
                test = undefined;
            }
            history = BatchExecutor.summary[route];
        }
        return {
            isBusy: (test && test.length > 0),
            history: history,
        };
    };
    /***
     * supprime un batch de la liste des batch en cours d'execution.
     * @param {Batch} batch une route donnée.
     */
    BatchExecutor.prototype.removeBatch = function (batch) {
        var unit = batch.unit;
        var criteria = { _id: "_" + unit.id };
        // let batch = _.find(BatchExecutor.processing[ unit.route ], criteria);
        if (batch) {
            BatchExecutor.Instance.addToSummary(batch);
            _.remove(BatchExecutor.processing[unit.route], batch);
            if (BatchExecutor.processing[unit.route].length === 0) {
                delete BatchExecutor.processing[unit.route];
            }
        }
    };
    /***
     * Lance un Batch
     * @param unit un BatchUnit see{@link BatchUnit}
     * @return renvoie une promesse du BatchUnit traité
     */
    BatchExecutor.prototype.runBatch = function (unit) {
        var criteria = { _id: "_" + unit.id };
        var batch = _.find(BatchExecutor.processing[unit.route], criteria);
        if (!batch) {
            batch = _.find(BatchExecutor.queue[unit.route], criteria);
            batch.status = batch_status_1.STATUS.RUNNING;
            if (!(BatchExecutor.processing[unit.route] instanceof Array)) {
                BatchExecutor.processing[unit.route] = [];
            }
            BatchExecutor.processing[unit.route].push(batch);
            _.remove(BatchExecutor.queue[unit.route], batch);
            if (BatchExecutor.queue[unit.route].length === 0) {
                delete BatchExecutor.queue[unit.route];
            }
            return batch.run();
        }
        else {
            return promise_api_1.Promise.resolve(unit);
        }
    };
    /***
     * Contient la liste des batchs en cours de traitement
     * @instance
     */
    BatchExecutor.processing = {};
    /***
     * Contient la liste des batchs dans la file d'attente
     * @instance
     */
    BatchExecutor.queue = {};
    /***
     * Contient l'historique la liste des batchs qui ont été executés la durée de vie est la
     * même que l'instance du serveur [aucune persistance]
     * @instance
     */
    BatchExecutor.summary = {};
    return BatchExecutor;
}());
exports.BatchExecutor = BatchExecutor;

//# sourceMappingURL=batch-executor.js.map
