"use strict";
/**
 * Copyright ou © ou Copr. Ministère de l'Europe et des Affaires étrangères (2017)
 * <p/>
 * pole-architecture.dga-dsi-psi@diplomatie.gouv.fr
 * <p/>
 * Ce logiciel est un programme informatique servant à faciliter la création
 * d'applications Web conformément aux référentiels généraux français : RGI, RGS et RGAA
 * <p/>
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 * <p/>
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 * <p/>
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 * <p/>
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 * <p/>
 * <p/>
 * Copyright or © or Copr. Ministry for Europe and Foreign Affairs (2017)
 * <p/>
 * pole-architecture.dga-dsi-psi@diplomatie.gouv.fr
 * <p/>
 * This software is a computer program whose purpose is to facilitate creation of
 * web application in accordance with french general repositories : RGI, RGS and RGAA.
 * <p/>
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 * <p/>
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 * <p/>
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 * <p/>
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * hornet-js-batch - Ensemble des composants de gestion de base hornet-js
 *
 * @author MEAE - Ministère de l'Europe et des Affaires étrangères
 * @version v5.2.3
 * @link git+https://github.com/diplomatiegouvfr/hornet-js.git
 * @license CECILL-2.1
 */
var hornet_js_utils_1 = require("hornet-js-utils");
var transformer_1 = require("hornet-js-batch/src/core/transform/transformer");
var batch_executor_1 = require("hornet-js-batch/src/core/batch-executor");
var filter_1 = require("hornet-js-batch/src/core/filter/filter");
var foreach_1 = require("hornet-js-batch/src/core/service/foreach");
var batch_status_1 = require("hornet-js-batch/src/core/batch-status");
var call_1 = require("hornet-js-batch/src/core/service/call");
var uuid = require("uuid");
var logger = hornet_js_utils_1.Utils.getLogger("hornet-js-batch.batch-unit");
/***
 * @classdesc Classe BatchUnit
 * C'est un helper qui permet de simplifier l'écriture des batchs pour l'utilisateur
 * @class
 */
var BatchUnit = /** @class */ (function () {
    function BatchUnit(name, model, route) {
        this.model = model;
        this.route = route;
        this._id = uuid.v4();
        this._name = name;
        this.status = batch_status_1.STATUS.INIT;
    }
    Object.defineProperty(BatchUnit.prototype, "id", {
        /***
         * renvoie l'id du batchUnit
         */
        get: function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BatchUnit.prototype, "name", {
        /***
         * renvoie le nom du batchUnit
         * @returns {string}
         */
        get: function () {
            return this._name;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BatchUnit.prototype, "status", {
        /***
         * renvoie le status du batchUnit
         */
        get: function () {
            return this._status;
        },
        /***
         * modifie le status batchUnit
         */
        set: function (status) {
            this._status = status;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Methode qui ajoute un ou plusieurs reader au traitement
     *  Les `readers` sont des classes de type BatchReader qui vont récupérer des données.
     * @param  {BatchReader} readers la liste des readers à lancer
     * @returns l'instance batchUnit {BatchUnit} en cours
     */
    BatchUnit.prototype.reader = function () {
        var readers = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            readers[_i] = arguments[_i];
        }
        batch_executor_1.BatchExecutor.Instance.getBatch(this).roadmap.push(readers);
        return this;
    };
    /**
     * Methode qui ajoute un ou plusieurs writer au traitement.
     * Les `writers` sont des classes de type BatchWriter qui vont renvoyer les données.
     * @param  {BatchReader} readers la liste des writers à lancer
     * @returns l'instance batchUnit {BatchUnit} en cours
     */
    BatchUnit.prototype.writer = function () {
        var writers = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            writers[_i] = arguments[_i];
        }
        batch_executor_1.BatchExecutor.Instance.getBatch(this).roadmap.push(writers);
        return this;
    };
    /**
     * Methode qui ajoute un filter au traitement.
     * Les `filters` sont des classes de type BatchFilter qui vont filtrer les données.
     * @param  {Function} filter la function filter à lancer
     * @returns l'instance batchUnit {BatchUnit} en cours
     */
    BatchUnit.prototype.filter = function (filter) {
        var newFilter = new filter_1.Filter();
        newFilter.name = this._name + ".filter";
        var batch = batch_executor_1.BatchExecutor.Instance.getBatch(this);
        newFilter.options.service = filter;
        batch_executor_1.BatchExecutor.Instance.getBatch(this).roadmap.push(newFilter);
        return this;
    };
    /**
     * Methode qui ajoute un transformer au traitement.
     * Les `transformers` sont des classes de type BatchTransform qui vont manipuler les données.
     * @param {Function} transformer la fonction qui manipule les données
     * @returns l'instance batchUnit {BatchUnit} en cours
     */
    BatchUnit.prototype.transform = function (transformer) {
        var newTransformer = new transformer_1.Transformer();
        newTransformer.name = this._name + ".transform";
        newTransformer.options.service = transformer;
        batch_executor_1.BatchExecutor.Instance.getBatch(this).roadmap.push(newTransformer);
        return this;
    };
    /**
     * Methode qui ajoute un mapper au traitement.
     * Les `mappers` sont des classes de type BatchMapper qui vont binder les données.
     * @param {BatchMapper} mapper le mapper à lancer
     * @returns l'instance batchUnit {BatchUnit} en cours
     */
    BatchUnit.prototype.mapper = function (mapper) {
        batch_executor_1.BatchExecutor.Instance.getBatch(this).roadmap.push(mapper);
        return this;
    };
    /**
     * Methode qui ajoute un appel de service au traitement.
     * Les `call` sont des classes de type BatchService qui vont executer l'appel de service.
     * @param {Function} service le service
     * @param {RouteActionBatch|RouteActionBatchMulti} scope l'action de type batch
     * @returns l'instance batchUnit {BatchUnit} en cours
     */
    BatchUnit.prototype.call = function (service, scope) {
        var newCall = new call_1.Call();
        newCall.name = this._name + ".call";
        newCall.options.service = service;
        newCall.options.scope = scope;
        batch_executor_1.BatchExecutor.Instance.getBatch(this).roadmap.push(newCall);
        return this;
    };
    /**
     * Méthode qui ajoute un appel de service au traitement.
     * Les `foreach` sont des classes de type BatchService qui vont executer l'appel de service en bouclant
     * sur la liste des paramètres qu'ils ont en entrée.
     * @param {Function} service le serie d'appel au service (boucle sur les arguments)
     * @param {RouteActionBatch|RouteActionBatchMulti} scope l'action de type batch
     * @returns l'instance batchUnit {BatchUnit} en cours
     */
    BatchUnit.prototype.foreach = function (service, scope) {
        var newForEach = new foreach_1.ForEach();
        newForEach.name = this._name + ".foreach";
        newForEach.options.service = service;
        newForEach.options.scope = scope;
        batch_executor_1.BatchExecutor.Instance.getBatch(this).roadmap.push(newForEach);
        return this;
    };
    /**
    * Lance le traitement du batch unit
    * @returns une promesse de type {@see BatchUnit} traité.
    */
    BatchUnit.prototype.run = function () {
        return batch_executor_1.BatchExecutor.Instance.runBatch(this);
    };
    return BatchUnit;
}());
exports.BatchUnit = BatchUnit;

//# sourceMappingURL=batch-unit.js.map
